//
//  const.h
//  SEM_movement
//
//  Created by Alexander Gord on 7/5/11.
//  Last updated 6/11/2014
//  Copyright 2011-2014
//


#ifndef const_h
#define const_h


// Comment this out to run on the gpu
#define RUN_ON_CPU

//These options will cause cell to delaminate if they cell centers drift to far 
// from the membrane as defined by DELAMINATION_RANGE
#define DELAMINATION
#define DELAMINATION_RANGE 5.5
//#define TIME_TO_DETACHMENT
// comment out the next line to prevent the number of cells of each type from being output to cell_out.txt at each time step
#define CELL_NUMBER_OUTPUT
// comment out the next line to prevent cell center data from being written to center_output.txt at each time step
#define CELL_CENTER_OUTPUT

// CELL_GROWTH, CPU_CELL_GROWTH, and EMBRYO_DIVISION ARE mutually exclusive, only set one
//#define CELL_GROWTH
// the cell growth code used by CELL_GROWTH is outdated, use CPU_CELL_GROWTH below
#define CPU_CELL_GROWTH
// GLOBAL_GROWTH_SWITCH causes asymmetric division to be based on the total number of basal cells, rather than the local density of basal cells
// if asymmetric division is not desired, set MAX_BASAL_CELLS to be larger than the number of cells that will exist in the simulation
// this is fairly clunky, sorry
#define GLOBAL_GROWTH_SWITCH
#define MAX_BASAL_CELLS 100
// SUPRA_BASAL_GROWTH turns on growth and division in suprabasal cells
#define SUPRA_BASAL_GROWTH

// Set how many movement time steps there are between growth time steps
#define ITERATIONS_BETWEEN_GROWTH_CYCLES    1000

// Set options for either CPU or GPU
#ifdef RUN_ON_CPU
#define DEFAULT_LOCAL_WORK_SIZE 1
#define COMPUTE_DEVICE CL_DEVICE_TYPE_CPU
#define DEFAULT_PROGRAM    "rk2_cpu.cl"
#define DEFAULT_NO_INTERACTION_PROGRAM  "no_interaction_cpu.cl"
#else
#define DEFAULT_LOCAL_WORK_SIZE 512
#define COMPUTE_DEVICE CL_DEVICE_TYPE_GPU
#define DEFAULT_PROGRAM    "rk2_gpu.cl"
#define DEFAULT_NO_INTERACTION_PROGRAM  "no_interaction_gpu.cl"
#endif

// here are some default values
#define DEFAULT_TIME_STEP       0.01
#define DEFAULT_ITERATIONS      100
#define DEFAULT_ELEMENT_TYPES   2
#define DEFAULT_MAX_ELEMENTS    32

//define the default values for the morse potential. These are legacy values as the morse potential is no longer used
#define DEFAULT_NUM_PARAMS      8

#define DEFAULT_U_INTRA         0.3
#define DEFAULT_UE_INTRA        0.1
#define DEFAULT_V_INTRA         0.12
#define DEFAULT_VE_INTRA        0.36

#define DEFAULT_U_INTER         0.3
#define DEFAULT_UE_INTER        0.05
#define DEFAULT_V_INTER         0.12
#define DEFAULT_VE_INTER        0.24

//define default program
#define DEFAULT_POTENTIAL         "morse.cl"









#endif
