// This is the kernel to run cell_growth
__kernel void cell_growth( int *acell, float *ax, float *ay, float *az, const int an, int *bcell, float *bx, float *by, float *bz, int bn, int *num_grown_elements)
{
//acell is the vector containing which cell each element belongs to
//ax, ay, az are vectors containing the position of one set of sub cell elements
//bx, by, bz are vectors containing the position of the other set of sub cell elements
//bn is the number of cells there are of type b. The number of type a are given by the work size.
//shared is the shared memory used by the kernel
//params is a constant array containing the pramateres used by the kernel
//num_params in the number of parameters used by the kernel

    int gId=get_global_id(0); //get the global ID of this work unit. This will be the cell number of interest
    
    int cx = 0; //Calculate the center of the top of the stem cell in the x and y direction 
    int cy = 0;
    int mz = -200000; //Find the maximum z value of the stem cell
    int n = 0; // count the number of elements in this cell
    
    for( int i = 0; i < an; i++)
    {
        if(acell[i]==gId)
        {
            cx=cx+ax[i];
            cy=cy+ay[i];
            if(az[i]>mz) mz=az[i];
            n++;
        }
    }

    for( int i = 0; i < bn; i++)
    {
        if(bcell[i]==gId)
        {
            cx=cx+bx[i];
            cy=cy+by[i];
            if(bz[i]>mz) mz=bz[i];
            n++;
        }
    }
    
    cx=cx/n;
    cy=cy/n;
    
    bx[bn+gId]=cx;
    by[bn+gId]=cy;
    bz[bn+gId]=mz+.5;
    bcell[bn+gId]=gId;
    num_grown_elements[gId]++;
   
    
}
