//the intracellular potential, the morse potential in this case
float intra_potential(float r, __constant float * params, int num_params, float x, float y, float z, float * x_F1, float * y_F1, float * z_F1)
{
    if(r<=0) return 0;
    float rm = (2.2f)/r;
    float m = 1.0f*((rm*rm*rm*rm*rm*rm*rm*rm*rm*rm*rm*rm)*(1/(r*r))-(rm*rm*rm));
    m -= .01*r;
    if(m*m>100) return 10.0f;
    else return 1.0f*m;
}


//the intercellular potential, the repulsive force morse potential in this case
float inter_potential(float r, __constant float * params, int num_params, float x, float y, float z, float * x_F1, float * y_F1, float * z_F1)
{
    if(r > 3.5) return 0;
    if(r<=0) return 0;
    float rm = 2.5f/r;
    float m = 0.001f*((rm*rm*rm*rm*rm*rm*rm*rm*rm*rm*rm*rm)*(1/(r*r))-(rm*rm*rm*rm*rm*rm)*(1/(r*r)));
    if(m*m<100) return 1.0f*m;
    else return 10.0f;
}

void external_potential( __constant float * params, int num_params, float x, float y, float z, float * x_F1, float * y_F1, float * z_F1)
{
//The basement membrane is actually located at z=-1, so that the cell centers can be located at z=0;

    float r = z+1.0f;
    float m = 0;
    if( (r > 10.5) || (r <= 0))
    {
        m=0;
    }
    else
    {
        float rm = 2.5f/r;
	//m = 30.0f*((rm*rm*rm*rm*rm*rm*rm*rm*rm*rm*rm*rm)*(1/(r*r))-1);
        m=-3.0f*z;
	if(m*m<10000)
        {
            m=m;
        }
        else
        {
            m = 100.0f;
        }
    }
    if( r <= 0 )
    {
        m=10.0f;
    }
    *z_F1+=m;


    if(x < 2.0f) *x_F1+=-(x-2);
    if(x > 48.0f) *x_F1+=48-x;
    if(y < 2.0f) *y_F1+=-(y-2);
    if(y > 48.0f) *y_F1+=48-y;


    return;
}
