// 3d distance function
float distance3( float x1, float y1, float z1, float x2, float y2, float z2 )
{
    return sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
}


// the first kernel for the RK2 method
/*
__kernel void rk2( __constant int *acell, __constant float *ax, __constant float *ay, __constant float *az, const int an, __constant int *bcell, __constant float *bx, __constant float *by, __constant float *bz, const int bn, const float dt, __local float * coordinates, __local int * cell_num, __global float* ax_F1, __global float* ay_F1, __global float *az_F1, __constant float * params, const int num_params)
*/
__kernel void rk2( __constant int *acell, __constant float *ax, __constant float *ay, __constant float *az, const int an, __constant int *bcell, __constant float *bx, __constant float *by, __constant float *bz, const int bn, const float dt, __local float * coordinates, __local int * cell_num, __global float* ax_F1, __global float* ay_F1, __global float *az_F1, __constant float * params, const int num_params)

{
//acell is the vector containing which cell each element belongs to
//ax, ay, az are vectors containing the position of one set of sub cell elements
//bx, by, bz are vectors containing the position of the other set of sub cell elements
//bn is the number of cells there are of type b. The number of type a are given by the work size.
//shared is the shared memory used by the kernel
//params is a constant array containing the pramateres used by the kernel
//num_params in the number of parameters used by the kernel


    int gId=get_global_id(0); //get the global ID of this work unit.
    int lSize = get_local_size(0); // get the work unit size.
    int lId=get_local_id(0); //get the local work ID.


//    float x = ax[gId]; //get the coordinates of the current element
//    float y = ay[gId];
//    float z = az[gId];


//    int local_cell_num = acell[gId]; // get the cell number of the current element

    float x_F1 = 0.0f; //initialize temporary local variables which will be added to the global values later
    float y_F1 = 0.0f;
    float z_F1 = 0.0f;
    
    //create a variable to store distance
    float r;


    float V; //a temporary variable to store potentials

    
    for(int i = 0; i+lSize < bn; i += lSize)
    {
        // copy data into shared memory for faster access
        coordinates[i+lId] = bx[i+lId];
        coordinates[i+lId+bn] = by[i+lId];
        coordinates[i+lId+2*bn] = bz[i+lId];
        
        cell_num[i+lId]=bcell[i+lId];
    
    }

    
    if(lId < bn%lSize)
    {   
        int i = (bn/lSize)*lSize;
        coordinates[i+lId] = bx[i+lId];
        coordinates[i+lId+bn] = by[i+lId];
        coordinates[i+lId+2*bn] = bz[i+lId];

        cell_num[i+lId]=bcell[i+lId];
    }
    barrier(CLK_LOCAL_MEM_FENCE);


    if(gId >= an) // This is necessary since we may need to pad the global size
    {
        return;
    }

    for(int i = 0; i < bn; i++)
    {
        r = distance3(ax[gId],ay[gId],az[gId],coordinates[i],coordinates[i+bn],coordinates[i+2*bn]);
        if(r==0)
        {
            external_potential(params,num_params,ax[gId],ay[gId],az[gId],&x_F1,&y_F1,&z_F1);
        }
        else
        {
            if(acell[gId]==cell_num[i])
            {
                V=intra_potential(r,params,num_params,ax[gId],ay[gId],az[gId],&x_F1,&y_F1,&z_F1);
            }
            else
            {
                V=inter_potential(r,params,num_params,ax[gId],ay[gId],az[gId],&x_F1,&y_F1,&z_F1);
            }
            if(V==V) //this will return false if V = NAN
            {
                x_F1 += V * (ax[gId]-coordinates[i]);
                y_F1 += V * (ay[gId]-coordinates[i+bn]);
                z_F1 += V * (az[gId]-coordinates[i+2*bn]);
            }
        }
    }
    
    ax_F1[gId]=ax_F1[gId]+.5 * dt * x_F1; //save the calculated values
    ay_F1[gId]=ay_F1[gId]+.5 * dt * y_F1;
    az_F1[gId]=az_F1[gId]+.5 * dt * z_F1;
}